"<!-- start footer part -->\n"
"<!--BEGIN GENERATE_TREEVIEW-->\n"
"<div id=\"nav-path\" class=\"navpath\"><!-- id is needed for treeview function! -->\n"
"  <ul>\n"
"    $navpath\n"
"    <li class=\"footer\">$generatedby\n"
"    <a href=\"http://www.doxygen.org/index.html\">\n"
"    <img class=\"footer\" src=\"$relpath^doxygen.png\" alt=\"doxygen\"/></a> $doxygenversion </li>\n"
"  </ul>\n"
"</div>\n"
"<!--END GENERATE_TREEVIEW-->\n"
"<!--BEGIN !GENERATE_TREEVIEW-->\n"
"<hr class=\"footer\"/><address class=\"footer\"><small>\n"
"$generatedby &#160;<a href=\"http://www.doxygen.org/index.html\">\n"
"<img class=\"footer\" src=\"$relpath^doxygen.png\" alt=\"doxygen\"/>\n"
"</a> $doxygenversion\n"
"</small></address>\n"
"<!--END !GENERATE_TREEVIEW-->\n"
"</body>\n"
"</html>\n"
